@@ -782,6 +782,11 @@ ngx_http_lua_ffi_req_get_headers_count(ngx_http_request_t *r, int max,
 {
     int                           count;
     ngx_list_part_t              *part;
+#if (NGX_HTTP_V3)
+    int                           has_host = 0;
+    ngx_uint_t                    i;
+    ngx_table_elt_t              *header;
+#endif
 
     if (r->connection->fd == (ngx_socket_t) -1) {
         return NGX_HTTP_LUA_FFI_BAD_CONTEXT;
@@ -794,11 +799,54 @@ ngx_http_lua_ffi_req_get_headers_count(ngx_http_request_t *r, int max,
     }
 
     part = &r->headers_in.headers.part;
+
+#if (NGX_HTTP_V3)
+    count = 0;
+    header = part->elts;
+
+    if (r->http_version == NGX_HTTP_VERSION_30
+        && r->headers_in.server.data != NULL)
+    {
+        has_host = 1;
+        count++;
+    }
+
+    if (has_host == 1) {
+        for (i = 0; /* void */; i++) {
+            if (i >= part->nelts) {
+                if (part->next == NULL) {
+                    break;
+                }
+
+                part = part->next;
+                header = part->elts;
+                i = 0;
+            }
+
+            if (header[i].key.len == 4
+                && ngx_strncasecmp(header[i].key.data,
+                                   (u_char *) "host", 4) == 0)
+            {
+                continue;
+            }
+
+            count++;
+        }
+
+    } else {
+        count = part->nelts;
+        while (part->next != NULL) {
+            part = part->next;
+            count += part->nelts;
+        }
+    }
+#else
     count = part->nelts;
     while (part->next != NULL) {
         part = part->next;
         count += part->nelts;
     }
+#endif
 
     if (max > 0 && count > max) {
         *truncated = 1;
@@ -821,12 +869,29 @@ ngx_http_lua_ffi_req_get_headers(ngx_http_request_t *r,
     ngx_uint_t                    i;
     ngx_list_part_t              *part;
     ngx_table_elt_t              *header;
+#if (NGX_HTTP_V3)
+    int                           has_host = 0;
+#endif
 
     if (count <= 0) {
         return NGX_OK;
     }
 
     n = 0;
+
+#if (NGX_HTTP_V3)
+    if (r->http_version == NGX_HTTP_VERSION_30
+        && r->headers_in.server.data != NULL)
+    {
+        out[n].key.data = (u_char *) "host";
+        out[n].key.len = sizeof("host") - 1;
+        out[n].value.len = r->headers_in.server.len;
+        out[n].value.data = r->headers_in.server.data;
+        has_host = 1;
+        ++n;
+    }
+#endif
+
     part = &r->headers_in.headers.part;
     header = part->elts;
 
@@ -842,6 +907,14 @@ ngx_http_lua_ffi_req_get_headers(ngx_http_request_t *r,
             i = 0;
         }
 
+#if (NGX_HTTP_V3)
+        if (has_host == 1 && header[i].key.len == 4
+            && ngx_strncasecmp(header[i].key.data, (u_char *) "host", 4) == 0)
+        {
+            continue;
+        }
+#endif
+
         if (raw) {
             out[n].key.data = header[i].key.data;
             out[n].key.len = (int) header[i].key.len;
@@ -1250,8 +1323,10 @@ ngx_http_lua_ngx_raw_header_cleanup(void *data)
 int
 ngx_http_lua_ffi_set_resp_header_macos(ngx_http_lua_set_resp_header_params_t *p)
 {
-    return ngx_http_lua_ffi_set_resp_header(p->r, p->key_data, p->key_len,
-                                            p->is_nil, p->sval, p->sval_len,
+    return ngx_http_lua_ffi_set_resp_header(p->r, (const u_char *) p->key_data,
+                                            p->key_len, p->is_nil,
+                                            (const u_char *) p->sval,
+                                            p->sval_len,
                                             p->mvals, p->mvals_len,
                                             p->override, p->errmsg);
 }
